from dingodb.protos.proxy_common_pb2 import *
from dingodb.protos.proxy_index_pb2 import *
from dingodb.protos.proxy_index_pb2_grpc import *
from dingodb.protos.proxy_meta_pb2 import *
from dingodb.protos.proxy_meta_pb2_grpc import *
from google.protobuf.json_format import MessageToDict, ParseDict

import grpc
from grpc._cython import cygrpc

from .grpc_dingo_param import (
    CheckClintParam,
    CheckCreateIndexParam,
    CheckVectorAddParam,
    CheckVectorDeleteParam,
    CheckVectorGetParam,
    CheckVectorScanParam,
    CheckVectorSearchParam,
)


class GrpcDingoDB:
    def __init__(self, host: list, timeout: int = 55000) -> None:
        CheckClintParam(host=host, timeout=timeout)
        self._get_channel(host, timeout)
        self.index_stub = IndexServiceStub(self._channel)
        self.meta_stub = MetaServiceStub(self._channel)

    def _get_channel(self, host, timeout):
        self.opts = [
            (cygrpc.ChannelArgKey.max_send_message_length, -1),
            (cygrpc.ChannelArgKey.max_receive_message_length, -1),
            ("grpc.enable_retries", 1),
            ("grpc.keepalive_time_ms", timeout),
        ]
        # channel_list

        self._channel = grpc.insecure_channel(host[0], options=self.opts)

    def __del__(self):
        self._channel.close()

    def close(self):
        self._channel.close()

    def create_index(
        self,
        index_name: str,
        dimension: int,
        index_type: str = "hnsw",
        metric_type: str = "cosine",
        replicas: int = 3,
        index_config: dict = None,
        metadata_config: dict = None,
        partition_rule: dict = None,
        operand: list = None,
        auto_id: bool = True,
        start_id: int = 1,
    ) -> bool:
        """
        create_index create index

        Args:
            index_name (str): the name of index
            dimension (int): dimension of vector
            index_type (str, optional): index type, one of {"flat", "hnsw"}. Defaults to "hnsw".
            metric_type (str, optional): metric type, one of {"dotproduct", "euclidean", "cosine"}. Defaults to "cosine"
            replicas (int, optional): dingoDB store replicas. Defaults to 3.
            index_config (dict, optional): Advanced configuration options for the index. Defaults to None.
            metadata_config (dict, optional): metadata. Defaults to None.
            partition_rule (dict, optional): partition rule. Defaults to None.
            operand (list, optional): operand. Defaults to None.
            auto_id (bool, optional): isAutoIncrement or not isAutoIncrement. Defaults to True.
            start_id (int, optional): autoIncrement start id. Defaults to 1.

        Raises:
            RuntimeError: return error

        Returns:
            bool: create table status
        """
        params = CheckCreateIndexParam(
            index_name=index_name,
            dimension=dimension,
            index_type=index_type,
            metric_type=metric_type,
            replicas=replicas,
            index_config=index_config,
            metadata_config=metadata_config,
            partition_rule=partition_rule,
            operand=operand,
            auto_id=auto_id,
            start_id=start_id,
        )
        rule = PartitionRule(columns=[])
        if params.partition_rule == {} and operand is not None and len(operand) != 0:
            rule.func_name = "RANGE"
            for item in operand:
                rule.details.append(
                    PartitionDetailDefinition(
                        operand=[str(item)], operator="", part_name=""
                    )
                )
        vec_create_request = CreateIndexRequest(
            schema_name="dingo",
            definition=IndexDefinition(
                auto_increment=params.start_id if params.auto_id else 0,
                name=params.index_name,
                replica=params.replicas,
                version=0,
                with_auto_increment=params.auto_id,
                index_partition=rule,
                index_parameter=IndexParameter(
                    index_type=INDEX_TYPE_VECTOR,
                    vector_index_parameter=ParseDict(
                        params.index_config, VectorIndexParameter()
                    ),
                ),
            ),
        )
        vec_create_response = self.meta_stub.CreateIndex.future(vec_create_request)

        return vec_create_response.result().state

    def delete_index(self, index_name: str) -> bool:
        """
        delete_index del/drop index

        Args:
            index_name (str): the name of index

        Raises:
            RuntimeError: return error

        Returns:
            bool: True/False
        """
        del_index_request = DeleteIndexRequest(
            schema_name="dingo", index_name=index_name
        )

        del_index_response = self.index_stub.DeleteIndex.future(del_index_request)
        return del_index_response.result().state

    def vector_add(
        self, index_name: str, datas: list, vectors: list, ids: list = None
    ) -> list:
        """
        vector_add add vector

        Args:
            index_name (str): the name of index
            datas (list): metadata list
            vectors (list): vector list
            ids (list, optional): id list. Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            list: add vector info in dingoDB
        """
        params = CheckVectorAddParam(
            index_name=index_name, datas=datas, vectors=vectors, ids=ids
        )

        vec_add_request = VectorAddRequest(
            schema_name="dingo", index_name=params.index_name
        )
        for i, v in enumerate(params.vectors):
            vec_grpc = VectorWithId()
            scalar_data_grpc = VectorScalarData()
            for key, value in params.datas[i].items():
                entry = scalar_data_grpc.scalar_data[key]
                entry.field_type = STRING
                field = entry.fields.add()
                field.string_data = value

            vec_grpc.vector.CopyFrom(
                Vector(dimension=len(v), float_values=v, value_type=FLOAT)
            )
            vec_grpc.scalar_data.CopyFrom(scalar_data_grpc)
            if ids is not None:
                vec_grpc.id = params.ids[i]
            vec_add_request.vectors.append(vec_grpc)

        vec_add_response = self.index_stub.VectorAdd.future(vec_add_request)

        return [
            MessageToDict(vec, including_default_value_fields=True)
            for vec in vec_add_response.result().vectors
        ]

    def vector_count(self, index_name: str):
        """
        vector_count count in index

        Args:
            index_name (str): the name of in index

        Returns:
            int: count num
        """
        vec_count_request = VectorGetRegionMetricsRequest(
            schema_name="dingo", index_name=index_name
        )
        vec_count_response = self.index_stub.VectorGetRegionMetrics.future(
            vec_count_request
        )
        records = MessageToDict(vec_count_response.result().metrics)

        return int(records.get("currentCount", 0)) - int(records.get("deletedCount", 0))

    def vector_scan(
        self,
        index_name: str,
        start_id: int,
        max_count: int = 1000,
        is_reverse: bool = False,
        with_scalar_data: bool = True,
        with_table_data: bool = True,
        with_vector_data: bool = True,
        fields: list = None,
        filter_scalar: dict = None,
    ) -> list:
        """
        vector_scan scan with start_id

        Args:
            index_name (str): the name of in index
            start_id (int): start id
            max_count (int, optional): max scan count. Defaults to 1000.
            is_reverse (bool, optional): whether reverse. Defaults to False.
            with_scalar_data (bool, optional): whether  with scalar info. Defaults to True.
            with_table_data (bool, optional): whether  with table info. Defaults to True.
            with_vector_data (bool, optional): whether with vector info. Defaults to False.
            fields (list, optional): fields for return . Defaults to [].
            filter_scalar (dict, optional): filter_scalar for return . Defaults to None.

        Raises:
            RuntimeError: return error

        Returns:
            list:  scan info list
        """
        params = CheckVectorScanParam(
            index_name=index_name,
            start_id=start_id,
            max_count=max_count,
            is_reverse=is_reverse,
            with_scalar_data=with_scalar_data,
            with_table_data=with_table_data,
            with_vector_data=with_vector_data,
            fields=fields,
            filter_scalar=filter_scalar,
        )

        vec_scan_request = VectorScanQueryRequest(
            schema_name="dingo",
            index_name=params.index_name,
            max_scan_count=params.max_count,
            vector_id_start=params.start_id,
            is_reverse_scan=params.is_reverse,
            without_vector_data=not params.with_vector_data,
            with_scalar_data=params.with_scalar_data,
            with_table_data=params.with_table_data,
            selected_keys=params.fields,
        )
        if params.filter_scalar:
            scalar_data_grpc = VectorScalarData()
            for key, value in params.filter_scalar.items():
                entry = scalar_data_grpc.scalar_data[key]
                entry.field_type = STRING
                field = entry.fields.add()
                field.string_data = value
            vec_scan_request.use_scalar_filter = True
            vec_scan_request.scalar_for_filter.CopyFrom(scalar_data_grpc)
        vec_scan_response = self.index_stub.VectorScanQuery.future(vec_scan_request)

        return [MessageToDict(vec) for vec in vec_scan_response.result().vectors]

    def get_index(self):
        """
        get_index get all index

        Raises:
            RuntimeError: return error

        Returns:
            list: all index list
        """

        get_index_name_request = GetIndexNamesRequest(schema_name="dingo")

        get_index_name_response = self.index_stub.GetIndexNames.future(
            get_index_name_request
        )

        return get_index_name_response.result().names

    def get_max_index_row(self, index_name: str, get_min: bool = False):
        """
        get_max_index_row get max id in index

        Args:
            index_name (str): the name of in index
            get_min (bool): whether get min value


        Raises:
            RuntimeError: return error

        Returns:
            int: max id value
        """
        vec_max_id_request = VectorGetBorderIdRequest(
            schema_name="dingo", index_name=index_name, get_min=get_min
        )

        vec_max_id_response = self.index_stub.VectorGetBorderId.future(
            vec_max_id_request
        )

        return vec_max_id_response.result().id

    def vector_search(
        self,
        index_name: str,
        xq: list,
        top_k: int = 10,
        search_params: dict = None,
        pre_filter: bool = True,
    ) -> list:
        """
        vector_search search vector

        Args:
            index_name (str): the name of the index
            xq (list): query vector, List[float] or List[List[float]]
            top_k (int, optional): top k search. Defaults to 10.
            search_params (dict, optional): search params for index. Defaults to None.
            pre_filter (bool, optional): filter type for index, False is post-filter. Defaults to True.

        Raises:
            RuntimeError: return error

        Returns:
            List[dict]: search results
        """
        params = CheckVectorSearchParam(
            index_name=index_name,
            xq=xq,
            top_k=top_k,
            pre_filter=pre_filter,
            search_params=search_params,
        )

        vec_search_response = self.index_stub.VectorSearch.future(params.search_params)

        return [
            MessageToDict(vec, including_default_value_fields=True)
            for vec in vec_search_response.result().batch_results
        ]

    def vector_get(
        self, index_name: str, ids: list, scalar: bool = True, vector: bool = True
    ) -> list:
        """
        vector_get query vector

        Args:
            index_name (str): the name of the index
            ids (list): query id list
            scalar (bool, optional): res with or without scalar. Defaults to True.
            vector (bool, optional): res with or without vector. Defaults to True.

        Raises:
            RuntimeError: _description_

        Returns:
            list: _description_
        """
        params = CheckVectorGetParam(
            index_name=index_name, ids=ids, scalar=scalar, vector=vector
        )

        vec_get_request = VectorGetRequest(
            schema_name="dingo",
            index_name=params.index_name,
            vector_ids=params.ids,
            with_out_vector_data=not vector,
            with_scalar_data=scalar,
        )

        vec_get_response = self.index_stub.VectorGet.future(vec_get_request)

        return [MessageToDict(vec) for vec in vec_get_response.result().vectors]

    def vector_delete(self, index_name: str, ids: list):
        """
        vector_delete delete vector with ids

        Args:
            index_name (str): the name of the index
            ids (list): id list

        Raises:
            RuntimeError: return error

        Returns:
            list : [True, False, ...]
        """
        params = CheckVectorDeleteParam(index_name=index_name, ids=ids)

        vec_del_request = VectorDeleteRequest(
            schema_name="dingo", index_name=params.index_name, ids=ids
        )

        vec_del_response = self.index_stub.VectorDelete.future(vec_del_request)
        return vec_del_response.result().key_states



import warnings
from typing import List

from dingodb.protos.proxy_common_pb2 import *
from dingodb.protos.proxy_index_pb2 import *
from pydantic import BaseModel, validator

from . import grpc_config as config


class CheckClintParam(BaseModel):
    host: List[str]
    timeout: int


class CheckCreateIndexParam(BaseModel):
    index_name: str
    dimension: int
    index_type: str = "hnsw"
    metric_type: str = "euclidean"
    replicas: int = 3
    index_config: dict = None
    metadata_config: dict = None
    partition_rule: dict = None
    operand: list = None
    auto_id: bool = True
    start_id: int = 1

    @validator("index_type", always=True)
    def check_index_type(cls, value):
        if value not in config.index_config.keys():
            raise Exception(f"index_type  must in {list(config.index_config.keys())}")
        return value

    @validator("metric_type", always=True)
    def check_metric_type(cls, value):
        if value not in config.metric_type.keys():
            raise Exception(f"metric_type  must in {list(config.metric_type.keys())}")
        value = config.metric_type[value]
        return value

    @validator("replicas", always=True)
    def check_replicas(cls, value):
        if value < 0:
            raise Exception(f"{value} must >= 0")
        return value

    @validator("index_config", always=True)
    def check_index_config(cls, value, values):
        index_type = values.get("index_type")
        dimension = values.get("dimension")
        metric_type = values.get("metric_type")
        vector_index_parameter = config.index_config[index_type]

        if value is not None:
            index_keys = vector_index_parameter[
                list(vector_index_parameter.keys())[1]
            ].keys()
            for key, v in value.items():
                if key in index_keys:
                    vector_index_parameter[list(vector_index_parameter.keys())[1]][
                        key
                    ] = v
                    if key == "efConstruction" and (v > 500 or v < 100):
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        warnings.warn(f"efConstruction: {v} suggestion in 100-500")
                    if key == "maxElements" and (v > 1000000000 or v < 50000):
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        warnings.warn(f"maxElements:{v} suggestion in 50000-1000000000")
                    if key == "nlinks" and (v > 64 or v < 16):
                        if v < 0:
                            raise Exception(f"{key} must >= 0")
                        warnings.warn(f"nlinks:{v} suggestion in 16-64")
                else:
                    warnings.warn(f"index_config {key} not in {index_type}")

        vector_index_parameter[list(vector_index_parameter.keys())[1]][
            "dimension"
        ] = dimension
        vector_index_parameter[list(vector_index_parameter.keys())[1]][
            "metricType"
        ] = metric_type
        return vector_index_parameter

    @validator("partition_rule", always=True)
    def check_partition_rule(cls, value):
        value = {} if value is None else value
        return value


class CheckVectorAddParam(BaseModel):
    index_name: str
    vectors: List[list]
    datas: List[dict] = None
    ids: List[int] = None

    @validator("datas", always=True)
    def check_datas(cls, value, values):
        value = [{}] * len(values.get("vectors")) if value is None else value
        return value

    @validator("ids", always=True)
    def check_ids(cls, value, values):
        if value is None:
            assert len(values.get("datas")) == len(
                values.get("vectors")
            ), f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} is not equal"
        else:
            assert (
                len(values.get("datas")) == len(values.get("vectors")) == len(value)
            ), f"length datas:{len(values.get('datas'))} vectors: {len(values.get('vectors'))} ids:{len(value)} is not equal"
        return value


class CheckVectorScanParam(BaseModel):
    index_name: str
    start_id: int
    max_count: int = 1000
    is_reverse: bool = False
    with_scalar_data: bool = True
    with_table_data: bool = True
    with_vector_data: bool = True
    fields: list = None
    filter_scalar: dict = None

    @validator("*", always=True)
    def check_input(cls, value, field):
        if field.name == "start_id":
            if not value > 0:
                raise Exception("start_id must > 0")
        if field.name == "max_count":
            if not value > 0:
                raise Exception("max_count must > 0")
        if field.name == "fields":
            if value is None:
                value = []

        return value


class CheckVectorSearchParam(BaseModel):
    index_name: str
    xq: list
    top_k: int = 10
    fields: list = None
    pre_filter: bool = True
    search_params: dict = None

    @validator("xq", always=True)
    def check_xq(cls, value):
        if not isinstance(value[0], list):
            value = [value]
        return value

    @validator("fields", always=True)
    def check_fields(cls, value):
        if value is None:
            value = []
        return value

    @validator("search_params", always=True)
    def check_search_params(cls, search_params, values):
        parameter = VectorSearchParameter(
            selected_keys=values.get("fields"), top_n=values.get("top_k")
        )
        parameter.with_out_vector_data = (
            False
            if search_params is None
            else search_params.get("withoutVectorData", False)
        )
        parameter.with_scalar_data = (
            True if search_params is None else search_params.get("withScalarData", True)
        )

        vec_search_request = VectorSearchRequest(
            schema_name="dingo", index_name=values.get("index_name")
        )
        use_scalar_filter = False
        if search_params is not None and "meta_expr" in search_params.keys():
            if search_params["meta_expr"] is not None:
                use_scalar_filter = True
                parameter.vector_filter = SCALAR_FILTER
                parameter.vector_filter_type = (
                    QUERY_PRE if values.get("pre_filter") else QUERY_POST
                )

        parameter.use_scalar_filter = use_scalar_filter
        # scalar_data_map = {}
        for xq in values.get("xq"):
            vec_with_id = VectorWithId(
                vector=Vector(dimension=len(xq), float_values=xq, value_type=FLOAT)
            )
            if use_scalar_filter:
                scalar_data_grpc = VectorScalarData()
                for key, value in search_params["meta_expr"].items():
                    entry = scalar_data_grpc.scalar_data[key]
                    entry.field_type = STRING
                    field = entry.fields.add()
                    field.string_data = value

                vec_with_id.scalar_data.CopyFrom(scalar_data_grpc)
            vec_search_request.vectors.append(vec_with_id)

        ef_search = 32 if search_params is None else search_params.get("efSearch", 32)
        assert ef_search >= 0, f"efSearch must >= 0, but get {ef_search}"

        parameter.hnsw.CopyFrom(SearchHNSWParam(efSearch=ef_search))
        if search_params and "parallelOnQueries" in search_params.keys():
            parameter.flat.CopyFrom(
                SearchFlatParam(
                    parallel_on_queries=0
                    if search_params is None
                    else search_params.get("parallelOnQueries", 0)
                )
            )

        vec_search_request.parameter.CopyFrom(parameter)

        return vec_search_request


class CheckVectorGetParam(BaseModel):
    index_name: str
    ids: List[int]
    scalar: bool = True
    vector: bool = True


class CheckVectorDeleteParam(BaseModel):
    index_name: str
    ids: List[int]



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proxy_common.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12proxy_common.proto\x12\x17\x64ingodb.pb.proxy.common\"\xbe\x01\n\x10\x43olumnDefinition\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x10\n\x08sql_type\x18\x02 \x01(\t\x12\x14\n\x0c\x65lement_type\x18\x03 \x01(\t\x12\x11\n\tprecision\x18\x04 \x01(\x05\x12\r\n\x05scale\x18\x05 \x01(\x05\x12\x10\n\x08nullable\x18\x06 \x01(\x08\x12\x12\n\nindexOfKey\x18\x07 \x01(\x05\x12\x17\n\x0fhas_default_val\x18\x08 \x01(\x08\x12\x13\n\x0b\x64\x65\x66\x61ult_val\x18\t \x01(\t\"\x80\x01\n\x06Vector\x12\x11\n\tdimension\x18\x01 \x01(\x05\x12\x36\n\nvalue_type\x18\x02 \x01(\x0e\x32\".dingodb.pb.proxy.common.ValueType\x12\x14\n\x0c\x66loat_values\x18\x03 \x03(\x02\x12\x15\n\rbinary_values\x18\x04 \x03(\x0c\"\xbb\x01\n\x10VectorScalarData\x12N\n\x0bscalar_data\x18\x01 \x03(\x0b\x32\x39.dingodb.pb.proxy.common.VectorScalarData.ScalarDataEntry\x1aW\n\x0fScalarDataEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x33\n\x05value\x18\x02 \x01(\x0b\x32$.dingodb.pb.proxy.common.ScalarValue:\x02\x38\x01\"9\n\x0fVectorTableData\x12\x11\n\ttable_key\x18\x01 \x01(\x0c\x12\x13\n\x0btable_value\x18\x02 \x01(\x0c\"\xc9\x01\n\x0cVectorWithId\x12\n\n\x02id\x18\x01 \x01(\x04\x12/\n\x06vector\x18\x02 \x01(\x0b\x32\x1f.dingodb.pb.proxy.common.Vector\x12>\n\x0bscalar_data\x18\x03 \x01(\x0b\x32).dingodb.pb.proxy.common.VectorScalarData\x12<\n\ntable_data\x18\x04 \x01(\x0b\x32(.dingodb.pb.proxy.common.VectorTableData\"\x9f\x01\n\x12VectorWithDistance\x12=\n\x0evector_with_id\x18\x01 \x01(\x0b\x32%.dingodb.pb.proxy.common.VectorWithId\x12\x10\n\x08\x64istance\x18\x02 \x01(\x02\x12\x38\n\x0bmetric_type\x18\x03 \x01(\x0e\x32#.dingodb.pb.proxy.common.MetricType\"\xe6\x01\n\x0eIndexParameter\x12\x36\n\nindex_type\x18\x01 \x01(\x0e\x32\".dingodb.pb.proxy.common.IndexType\x12M\n\x16vector_index_parameter\x18\x02 \x01(\x0b\x32-.dingodb.pb.proxy.common.VectorIndexParameter\x12M\n\x16scalar_index_parameter\x18\x03 \x01(\x0b\x32-.dingodb.pb.proxy.common.ScalarIndexParameter\"^\n\x0f\x43reateFlatParam\x12\x11\n\tdimension\x18\x01 \x01(\r\x12\x38\n\x0bmetric_type\x18\x02 \x01(\x0e\x32#.dingodb.pb.proxy.common.MetricType\"u\n\x12\x43reateIvfFlatParam\x12\x11\n\tdimension\x18\x01 \x01(\r\x12\x38\n\x0bmetric_type\x18\x02 \x01(\x0e\x32#.dingodb.pb.proxy.common.MetricType\x12\x12\n\nncentroids\x18\x03 \x01(\x05\"\xba\x01\n\x10\x43reateIvfPqParam\x12\x11\n\tdimension\x18\x01 \x01(\r\x12\x38\n\x0bmetric_type\x18\x02 \x01(\x0e\x32#.dingodb.pb.proxy.common.MetricType\x12\x12\n\nncentroids\x18\x03 \x01(\x05\x12\x12\n\nnsubvector\x18\x04 \x01(\x05\x12\x18\n\x10\x62ucket_init_size\x18\x05 \x01(\x05\x12\x17\n\x0f\x62ucket_max_size\x18\x06 \x01(\x05\"\x9c\x01\n\x0f\x43reateHnswParam\x12\x11\n\tdimension\x18\x01 \x01(\r\x12\x38\n\x0bmetric_type\x18\x02 \x01(\x0e\x32#.dingodb.pb.proxy.common.MetricType\x12\x16\n\x0e\x65\x66\x43onstruction\x18\x03 \x01(\r\x12\x14\n\x0cmax_elements\x18\x04 \x01(\r\x12\x0e\n\x06nlinks\x18\x05 \x01(\x05\"\xa0\x01\n\x12\x43reateDiskAnnParam\x12\x11\n\tdimension\x18\x01 \x01(\r\x12\x38\n\x0bmetric_type\x18\x02 \x01(\x0e\x32#.dingodb.pb.proxy.common.MetricType\x12\x11\n\tnum_trees\x18\x03 \x01(\x05\x12\x15\n\rnum_neighbors\x18\x04 \x01(\x05\x12\x13\n\x0bnum_threads\x18\x05 \x01(\x05\"\xd9\x03\n\x14VectorIndexParameter\x12\x43\n\x11vector_index_type\x18\x01 \x01(\x0e\x32(.dingodb.pb.proxy.common.VectorIndexType\x12\x42\n\x0e\x66lat_parameter\x18\x02 \x01(\x0b\x32(.dingodb.pb.proxy.common.CreateFlatParamH\x00\x12I\n\x12ivf_flat_parameter\x18\x03 \x01(\x0b\x32+.dingodb.pb.proxy.common.CreateIvfFlatParamH\x00\x12\x45\n\x10ivf_pq_parameter\x18\x04 \x01(\x0b\x32).dingodb.pb.proxy.common.CreateIvfPqParamH\x00\x12\x42\n\x0ehnsw_parameter\x18\x05 \x01(\x0b\x32(.dingodb.pb.proxy.common.CreateHnswParamH\x00\x12H\n\x11\x64iskann_parameter\x18\x06 \x01(\x0b\x32+.dingodb.pb.proxy.common.CreateDiskAnnParamH\x00\x42\x18\n\x16vector_index_parameter\"\xc8\x01\n\x0cVectorSchema\x12\x38\n\x04type\x18\x01 \x01(\x0e\x32*.dingodb.pb.proxy.common.VectorSchema.Type\x12\x0e\n\x06is_key\x18\x02 \x01(\x08\x12\x13\n\x0bis_nullable\x18\x03 \x01(\x08\x12\r\n\x05index\x18\x04 \x01(\x05\"J\n\x04Type\x12\x08\n\x04\x42OOL\x10\x00\x12\x0b\n\x07INTEGER\x10\x01\x12\t\n\x05\x46LOAT\x10\x02\x12\x08\n\x04LONG\x10\x03\x12\n\n\x06\x44OUBLE\x10\x04\x12\n\n\x06STRING\x10\x05\"[\n\x14ScalarIndexParameter\x12\x43\n\x11scalar_index_type\x18\x01 \x01(\x0e\x32(.dingodb.pb.proxy.common.ScalarIndexType\".\n\x0fSearchFlatParam\x12\x1b\n\x13parallel_on_queries\x18\x01 \x01(\x05\"A\n\x12SearchIvfFlatParam\x12\x0e\n\x06nprobe\x18\x01 \x01(\x05\x12\x1b\n\x13parallel_on_queries\x18\x02 \x01(\x05\"S\n\x10SearchIvfPqParam\x12\x0e\n\x06nprobe\x18\x01 \x01(\x05\x12\x1b\n\x13parallel_on_queries\x18\x02 \x01(\x05\x12\x12\n\nrecall_num\x18\x03 \x01(\x05\"#\n\x0fSearchHNSWParam\x12\x10\n\x08\x65\x66Search\x18\x01 \x01(\x05\"\x14\n\x12SearchDiskAnnParam\"\xc6\x05\n\x15VectorSearchParameter\x12\r\n\x05top_n\x18\x01 \x01(\r\x12\x1c\n\x14with_out_vector_data\x18\x02 \x01(\x08\x12\x18\n\x10with_scalar_data\x18\x03 \x01(\x08\x12\x15\n\rselected_keys\x18\x04 \x03(\t\x12\x17\n\x0fwith_table_data\x18\x05 \x01(\x08\x12\x38\n\x04\x66lat\x18\x0b \x01(\x0b\x32(.dingodb.pb.proxy.common.SearchFlatParamH\x00\x12?\n\x08ivf_flat\x18\x0c \x01(\x0b\x32+.dingodb.pb.proxy.common.SearchIvfFlatParamH\x00\x12;\n\x06ivf_pq\x18\r \x01(\x0b\x32).dingodb.pb.proxy.common.SearchIvfPqParamH\x00\x12\x38\n\x04hnsw\x18\x0e \x01(\x0b\x32(.dingodb.pb.proxy.common.SearchHNSWParamH\x00\x12>\n\x07\x64iskann\x18\x0f \x01(\x0b\x32+.dingodb.pb.proxy.common.SearchDiskAnnParamH\x00\x12\x19\n\x11use_scalar_filter\x18\x14 \x01(\x08\x12<\n\rvector_filter\x18\x15 \x01(\x0e\x32%.dingodb.pb.proxy.common.VectorFilter\x12\x45\n\x12vector_filter_type\x18\x16 \x01(\x0e\x32).dingodb.pb.proxy.common.VectorFilterType\x12\x46\n\x12vector_coprocessor\x18\x17 \x01(\x0b\x32*.dingodb.pb.proxy.common.VectorCoprocessor\x12\x12\n\nvector_ids\x18\x18 \x03(\x04\x42\x08\n\x06search\"\xa1\x01\n\x11VectorCoprocessor\x12\x16\n\x0eschema_version\x18\x01 \x01(\x05\x12\x45\n\x0foriginal_schema\x18\x02 \x01(\x0b\x32,.dingodb.pb.proxy.common.VectorSchemaWrapper\x12\x19\n\x11selection_columns\x18\x03 \x03(\x05\x12\x12\n\nexpression\x18\x04 \x01(\x0c\"c\n\x13VectorSchemaWrapper\x12\x39\n\x06schema\x18\x01 \x03(\x0b\x32).dingodb.pb.proxy.common.ColumnDefinition\x12\x11\n\tcommon_id\x18\x02 \x01(\x03\"\xad\x01\n\x0bScalarField\x12\x13\n\tbool_data\x18\x01 \x01(\x08H\x00\x12\x12\n\x08int_data\x18\x02 \x01(\x05H\x00\x12\x13\n\tlong_data\x18\x03 \x01(\x03H\x00\x12\x14\n\nfloat_data\x18\x04 \x01(\x02H\x00\x12\x15\n\x0b\x64ouble_data\x18\x05 \x01(\x01H\x00\x12\x15\n\x0bstring_data\x18\x06 \x01(\tH\x00\x12\x14\n\nbytes_data\x18\x07 \x01(\x0cH\x00\x42\x06\n\x04\x64\x61ta\"\x81\x01\n\x0bScalarValue\x12<\n\nfield_type\x18\x01 \x01(\x0e\x32(.dingodb.pb.proxy.common.ScalarFieldType\x12\x34\n\x06\x66ields\x18\x02 \x03(\x0b\x32$.dingodb.pb.proxy.common.ScalarField\"\xbd\x01\n\x12VectorIndexMetrics\x12\x43\n\x11vector_index_type\x18\x01 \x01(\x0e\x32(.dingodb.pb.proxy.common.VectorIndexType\x12\x15\n\rcurrent_count\x18\x02 \x01(\x03\x12\x15\n\rdeleted_count\x18\x03 \x01(\x03\x12\x0e\n\x06max_id\x18\x04 \x01(\x03\x12\x0e\n\x06min_id\x18\x05 \x01(\x03\x12\x14\n\x0cmemory_bytes\x18\x06 \x01(\x03*!\n\tValueType\x12\t\n\x05\x46LOAT\x10\x00\x12\t\n\x05UINT8\x10\x01*e\n\x0fScalarIndexType\x12\x1a\n\x16SCALAR_INDEX_TYPE_NONE\x10\x00\x12\x19\n\x15SCALAR_INDEX_TYPE_LSM\x10\x01\x12\x1b\n\x17SCALAR_INDEX_TYPE_BTREE\x10\x02*N\n\tIndexType\x12\x13\n\x0fINDEX_TYPE_NONE\x10\x00\x12\x15\n\x11INDEX_TYPE_VECTOR\x10\x01\x12\x15\n\x11INDEX_TYPE_SCALAR\x10\x02*\xc2\x01\n\x0fVectorIndexType\x12\x1a\n\x16VECTOR_INDEX_TYPE_NONE\x10\x00\x12\x1a\n\x16VECTOR_INDEX_TYPE_FLAT\x10\x01\x12\x1e\n\x1aVECTOR_INDEX_TYPE_IVF_FLAT\x10\x02\x12\x1c\n\x18VECTOR_INDEX_TYPE_IVF_PQ\x10\x03\x12\x1a\n\x16VECTOR_INDEX_TYPE_HNSW\x10\x04\x12\x1d\n\x19VECTOR_INDEX_TYPE_DISKANN\x10\x05*m\n\nMetricType\x12\x14\n\x10METRIC_TYPE_NONE\x10\x00\x12\x12\n\x0eMETRIC_TYPE_L2\x10\x01\x12\x1d\n\x19METRIC_TYPE_INNER_PRODUCT\x10\x02\x12\x16\n\x12METRIC_TYPE_COSINE\x10\x03*I\n\x0cVectorFilter\x12\x11\n\rSCALAR_FILTER\x10\x00\x12\x10\n\x0cTABLE_FILTER\x10\x01\x12\x14\n\x10VECTOR_ID_FILTER\x10\n*1\n\x10VectorFilterType\x12\x0e\n\nQUERY_POST\x10\x00\x12\r\n\tQUERY_PRE\x10\x01*\x80\x01\n\x0fScalarFieldType\x12\x08\n\x04NONE\x10\x00\x12\x08\n\x04\x42OOL\x10\x01\x12\x08\n\x04INT8\x10\x02\x12\t\n\x05INT16\x10\x03\x12\t\n\x05INT32\x10\x04\x12\t\n\x05INT64\x10\x05\x12\x0b\n\x07\x46LOAT32\x10\x06\x12\n\n\x06\x44OUBLE\x10\x07\x12\n\n\x06STRING\x10\x08\x12\t\n\x05\x42YTES\x10\tB\x1c\n\x17io.dingodb.proxy.common\x88\x01\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'proxy_common_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\027io.dingodb.proxy.common\210\001\001'
  _VECTORSCALARDATA_SCALARDATAENTRY._options = None
  _VECTORSCALARDATA_SCALARDATAENTRY._serialized_options = b'8\001'
  _globals['_VALUETYPE']._serialized_start=4454
  _globals['_VALUETYPE']._serialized_end=4487
  _globals['_SCALARINDEXTYPE']._serialized_start=4489
  _globals['_SCALARINDEXTYPE']._serialized_end=4590
  _globals['_INDEXTYPE']._serialized_start=4592
  _globals['_INDEXTYPE']._serialized_end=4670
  _globals['_VECTORINDEXTYPE']._serialized_start=4673
  _globals['_VECTORINDEXTYPE']._serialized_end=4867
  _globals['_METRICTYPE']._serialized_start=4869
  _globals['_METRICTYPE']._serialized_end=4978
  _globals['_VECTORFILTER']._serialized_start=4980
  _globals['_VECTORFILTER']._serialized_end=5053
  _globals['_VECTORFILTERTYPE']._serialized_start=5055
  _globals['_VECTORFILTERTYPE']._serialized_end=5104
  _globals['_SCALARFIELDTYPE']._serialized_start=5107
  _globals['_SCALARFIELDTYPE']._serialized_end=5235
  _globals['_COLUMNDEFINITION']._serialized_start=48
  _globals['_COLUMNDEFINITION']._serialized_end=238
  _globals['_VECTOR']._serialized_start=241
  _globals['_VECTOR']._serialized_end=369
  _globals['_VECTORSCALARDATA']._serialized_start=372
  _globals['_VECTORSCALARDATA']._serialized_end=559
  _globals['_VECTORSCALARDATA_SCALARDATAENTRY']._serialized_start=472
  _globals['_VECTORSCALARDATA_SCALARDATAENTRY']._serialized_end=559
  _globals['_VECTORTABLEDATA']._serialized_start=561
  _globals['_VECTORTABLEDATA']._serialized_end=618
  _globals['_VECTORWITHID']._serialized_start=621
  _globals['_VECTORWITHID']._serialized_end=822
  _globals['_VECTORWITHDISTANCE']._serialized_start=825
  _globals['_VECTORWITHDISTANCE']._serialized_end=984
  _globals['_INDEXPARAMETER']._serialized_start=987
  _globals['_INDEXPARAMETER']._serialized_end=1217
  _globals['_CREATEFLATPARAM']._serialized_start=1219
  _globals['_CREATEFLATPARAM']._serialized_end=1313
  _globals['_CREATEIVFFLATPARAM']._serialized_start=1315
  _globals['_CREATEIVFFLATPARAM']._serialized_end=1432
  _globals['_CREATEIVFPQPARAM']._serialized_start=1435
  _globals['_CREATEIVFPQPARAM']._serialized_end=1621
  _globals['_CREATEHNSWPARAM']._serialized_start=1624
  _globals['_CREATEHNSWPARAM']._serialized_end=1780
  _globals['_CREATEDISKANNPARAM']._serialized_start=1783
  _globals['_CREATEDISKANNPARAM']._serialized_end=1943
  _globals['_VECTORINDEXPARAMETER']._serialized_start=1946
  _globals['_VECTORINDEXPARAMETER']._serialized_end=2419
  _globals['_VECTORSCHEMA']._serialized_start=2422
  _globals['_VECTORSCHEMA']._serialized_end=2622
  _globals['_VECTORSCHEMA_TYPE']._serialized_start=2548
  _globals['_VECTORSCHEMA_TYPE']._serialized_end=2622
  _globals['_SCALARINDEXPARAMETER']._serialized_start=2624
  _globals['_SCALARINDEXPARAMETER']._serialized_end=2715
  _globals['_SEARCHFLATPARAM']._serialized_start=2717
  _globals['_SEARCHFLATPARAM']._serialized_end=2763
  _globals['_SEARCHIVFFLATPARAM']._serialized_start=2765
  _globals['_SEARCHIVFFLATPARAM']._serialized_end=2830
  _globals['_SEARCHIVFPQPARAM']._serialized_start=2832
  _globals['_SEARCHIVFPQPARAM']._serialized_end=2915
  _globals['_SEARCHHNSWPARAM']._serialized_start=2917
  _globals['_SEARCHHNSWPARAM']._serialized_end=2952
  _globals['_SEARCHDISKANNPARAM']._serialized_start=2954
  _globals['_SEARCHDISKANNPARAM']._serialized_end=2974
  _globals['_VECTORSEARCHPARAMETER']._serialized_start=2977
  _globals['_VECTORSEARCHPARAMETER']._serialized_end=3687
  _globals['_VECTORCOPROCESSOR']._serialized_start=3690
  _globals['_VECTORCOPROCESSOR']._serialized_end=3851
  _globals['_VECTORSCHEMAWRAPPER']._serialized_start=3853
  _globals['_VECTORSCHEMAWRAPPER']._serialized_end=3952
  _globals['_SCALARFIELD']._serialized_start=3955
  _globals['_SCALARFIELD']._serialized_end=4128
  _globals['_SCALARVALUE']._serialized_start=4131
  _globals['_SCALARVALUE']._serialized_end=4260
  _globals['_VECTORINDEXMETRICS']._serialized_start=4263
  _globals['_VECTORINDEXMETRICS']._serialized_end=4452
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import proxy_meta_pb2 as proxy__meta__pb2


class MetaServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.CreateIndex = channel.unary_unary(
                '/dingodb.pb.proxy.meta.MetaService/CreateIndex',
                request_serializer=proxy__meta__pb2.CreateIndexRequest.SerializeToString,
                response_deserializer=proxy__meta__pb2.CreateIndexResponse.FromString,
                )
        self.UpdateIndex = channel.unary_unary(
                '/dingodb.pb.proxy.meta.MetaService/UpdateIndex',
                request_serializer=proxy__meta__pb2.UpdateIndexRequest.SerializeToString,
                response_deserializer=proxy__meta__pb2.UpdateIndexResponse.FromString,
                )
        self.UpdateMaxElements = channel.unary_unary(
                '/dingodb.pb.proxy.meta.MetaService/UpdateMaxElements',
                request_serializer=proxy__meta__pb2.UpdateMaxElementsRequest.SerializeToString,
                response_deserializer=proxy__meta__pb2.UpdateMaxElementsResponse.FromString,
                )
        self.DeleteIndex = channel.unary_unary(
                '/dingodb.pb.proxy.meta.MetaService/DeleteIndex',
                request_serializer=proxy__meta__pb2.DeleteIndexRequest.SerializeToString,
                response_deserializer=proxy__meta__pb2.DeleteIndexResponse.FromString,
                )
        self.GetIndex = channel.unary_unary(
                '/dingodb.pb.proxy.meta.MetaService/GetIndex',
                request_serializer=proxy__meta__pb2.GetIndexRequest.SerializeToString,
                response_deserializer=proxy__meta__pb2.GetIndexResponse.FromString,
                )
        self.GetIndexes = channel.unary_unary(
                '/dingodb.pb.proxy.meta.MetaService/GetIndexes',
                request_serializer=proxy__meta__pb2.GetIndexesRequest.SerializeToString,
                response_deserializer=proxy__meta__pb2.GetIndexesResponse.FromString,
                )
        self.GetIndexNames = channel.unary_unary(
                '/dingodb.pb.proxy.meta.MetaService/GetIndexNames',
                request_serializer=proxy__meta__pb2.GetIndexNamesRequest.SerializeToString,
                response_deserializer=proxy__meta__pb2.GetIndexNamesResponse.FromString,
                )


class MetaServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def CreateIndex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateIndex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def UpdateMaxElements(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteIndex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIndex(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIndexes(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetIndexNames(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_MetaServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'CreateIndex': grpc.unary_unary_rpc_method_handler(
                    servicer.CreateIndex,
                    request_deserializer=proxy__meta__pb2.CreateIndexRequest.FromString,
                    response_serializer=proxy__meta__pb2.CreateIndexResponse.SerializeToString,
            ),
            'UpdateIndex': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateIndex,
                    request_deserializer=proxy__meta__pb2.UpdateIndexRequest.FromString,
                    response_serializer=proxy__meta__pb2.UpdateIndexResponse.SerializeToString,
            ),
            'UpdateMaxElements': grpc.unary_unary_rpc_method_handler(
                    servicer.UpdateMaxElements,
                    request_deserializer=proxy__meta__pb2.UpdateMaxElementsRequest.FromString,
                    response_serializer=proxy__meta__pb2.UpdateMaxElementsResponse.SerializeToString,
            ),
            'DeleteIndex': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteIndex,
                    request_deserializer=proxy__meta__pb2.DeleteIndexRequest.FromString,
                    response_serializer=proxy__meta__pb2.DeleteIndexResponse.SerializeToString,
            ),
            'GetIndex': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIndex,
                    request_deserializer=proxy__meta__pb2.GetIndexRequest.FromString,
                    response_serializer=proxy__meta__pb2.GetIndexResponse.SerializeToString,
            ),
            'GetIndexes': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIndexes,
                    request_deserializer=proxy__meta__pb2.GetIndexesRequest.FromString,
                    response_serializer=proxy__meta__pb2.GetIndexesResponse.SerializeToString,
            ),
            'GetIndexNames': grpc.unary_unary_rpc_method_handler(
                    servicer.GetIndexNames,
                    request_deserializer=proxy__meta__pb2.GetIndexNamesRequest.FromString,
                    response_serializer=proxy__meta__pb2.GetIndexNamesResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'dingodb.pb.proxy.meta.MetaService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class MetaService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def CreateIndex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.meta.MetaService/CreateIndex',
            proxy__meta__pb2.CreateIndexRequest.SerializeToString,
            proxy__meta__pb2.CreateIndexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateIndex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.meta.MetaService/UpdateIndex',
            proxy__meta__pb2.UpdateIndexRequest.SerializeToString,
            proxy__meta__pb2.UpdateIndexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def UpdateMaxElements(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.meta.MetaService/UpdateMaxElements',
            proxy__meta__pb2.UpdateMaxElementsRequest.SerializeToString,
            proxy__meta__pb2.UpdateMaxElementsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteIndex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.meta.MetaService/DeleteIndex',
            proxy__meta__pb2.DeleteIndexRequest.SerializeToString,
            proxy__meta__pb2.DeleteIndexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIndex(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.meta.MetaService/GetIndex',
            proxy__meta__pb2.GetIndexRequest.SerializeToString,
            proxy__meta__pb2.GetIndexResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIndexes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.meta.MetaService/GetIndexes',
            proxy__meta__pb2.GetIndexesRequest.SerializeToString,
            proxy__meta__pb2.GetIndexesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetIndexNames(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.meta.MetaService/GetIndexNames',
            proxy__meta__pb2.GetIndexNamesRequest.SerializeToString,
            proxy__meta__pb2.GetIndexNamesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import proxy_index_pb2 as proxy__index__pb2


class IndexServiceStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.VectorAdd = channel.unary_unary(
                '/dingodb.pb.proxy.index.IndexService/VectorAdd',
                request_serializer=proxy__index__pb2.VectorAddRequest.SerializeToString,
                response_deserializer=proxy__index__pb2.VectorAddResponse.FromString,
                )
        self.VectorGet = channel.unary_unary(
                '/dingodb.pb.proxy.index.IndexService/VectorGet',
                request_serializer=proxy__index__pb2.VectorGetRequest.SerializeToString,
                response_deserializer=proxy__index__pb2.VectorGetResponse.FromString,
                )
        self.VectorSearch = channel.unary_unary(
                '/dingodb.pb.proxy.index.IndexService/VectorSearch',
                request_serializer=proxy__index__pb2.VectorSearchRequest.SerializeToString,
                response_deserializer=proxy__index__pb2.VectorSearchResponse.FromString,
                )
        self.VectorDelete = channel.unary_unary(
                '/dingodb.pb.proxy.index.IndexService/VectorDelete',
                request_serializer=proxy__index__pb2.VectorDeleteRequest.SerializeToString,
                response_deserializer=proxy__index__pb2.VectorDeleteResponse.FromString,
                )
        self.VectorGetBorderId = channel.unary_unary(
                '/dingodb.pb.proxy.index.IndexService/VectorGetBorderId',
                request_serializer=proxy__index__pb2.VectorGetBorderIdRequest.SerializeToString,
                response_deserializer=proxy__index__pb2.VectorGetBorderIdResponse.FromString,
                )
        self.VectorScanQuery = channel.unary_unary(
                '/dingodb.pb.proxy.index.IndexService/VectorScanQuery',
                request_serializer=proxy__index__pb2.VectorScanQueryRequest.SerializeToString,
                response_deserializer=proxy__index__pb2.VectorScanQueryResponse.FromString,
                )
        self.VectorGetRegionMetrics = channel.unary_unary(
                '/dingodb.pb.proxy.index.IndexService/VectorGetRegionMetrics',
                request_serializer=proxy__index__pb2.VectorGetRegionMetricsRequest.SerializeToString,
                response_deserializer=proxy__index__pb2.VectorGetRegionMetricsResponse.FromString,
                )


class IndexServiceServicer(object):
    """Missing associated documentation comment in .proto file."""

    def VectorAdd(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VectorGet(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VectorSearch(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VectorDelete(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VectorGetBorderId(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VectorScanQuery(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VectorGetRegionMetrics(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_IndexServiceServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'VectorAdd': grpc.unary_unary_rpc_method_handler(
                    servicer.VectorAdd,
                    request_deserializer=proxy__index__pb2.VectorAddRequest.FromString,
                    response_serializer=proxy__index__pb2.VectorAddResponse.SerializeToString,
            ),
            'VectorGet': grpc.unary_unary_rpc_method_handler(
                    servicer.VectorGet,
                    request_deserializer=proxy__index__pb2.VectorGetRequest.FromString,
                    response_serializer=proxy__index__pb2.VectorGetResponse.SerializeToString,
            ),
            'VectorSearch': grpc.unary_unary_rpc_method_handler(
                    servicer.VectorSearch,
                    request_deserializer=proxy__index__pb2.VectorSearchRequest.FromString,
                    response_serializer=proxy__index__pb2.VectorSearchResponse.SerializeToString,
            ),
            'VectorDelete': grpc.unary_unary_rpc_method_handler(
                    servicer.VectorDelete,
                    request_deserializer=proxy__index__pb2.VectorDeleteRequest.FromString,
                    response_serializer=proxy__index__pb2.VectorDeleteResponse.SerializeToString,
            ),
            'VectorGetBorderId': grpc.unary_unary_rpc_method_handler(
                    servicer.VectorGetBorderId,
                    request_deserializer=proxy__index__pb2.VectorGetBorderIdRequest.FromString,
                    response_serializer=proxy__index__pb2.VectorGetBorderIdResponse.SerializeToString,
            ),
            'VectorScanQuery': grpc.unary_unary_rpc_method_handler(
                    servicer.VectorScanQuery,
                    request_deserializer=proxy__index__pb2.VectorScanQueryRequest.FromString,
                    response_serializer=proxy__index__pb2.VectorScanQueryResponse.SerializeToString,
            ),
            'VectorGetRegionMetrics': grpc.unary_unary_rpc_method_handler(
                    servicer.VectorGetRegionMetrics,
                    request_deserializer=proxy__index__pb2.VectorGetRegionMetricsRequest.FromString,
                    response_serializer=proxy__index__pb2.VectorGetRegionMetricsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'dingodb.pb.proxy.index.IndexService', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class IndexService(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def VectorAdd(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.index.IndexService/VectorAdd',
            proxy__index__pb2.VectorAddRequest.SerializeToString,
            proxy__index__pb2.VectorAddResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VectorGet(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.index.IndexService/VectorGet',
            proxy__index__pb2.VectorGetRequest.SerializeToString,
            proxy__index__pb2.VectorGetResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VectorSearch(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.index.IndexService/VectorSearch',
            proxy__index__pb2.VectorSearchRequest.SerializeToString,
            proxy__index__pb2.VectorSearchResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VectorDelete(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.index.IndexService/VectorDelete',
            proxy__index__pb2.VectorDeleteRequest.SerializeToString,
            proxy__index__pb2.VectorDeleteResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VectorGetBorderId(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.index.IndexService/VectorGetBorderId',
            proxy__index__pb2.VectorGetBorderIdRequest.SerializeToString,
            proxy__index__pb2.VectorGetBorderIdResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VectorScanQuery(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.index.IndexService/VectorScanQuery',
            proxy__index__pb2.VectorScanQueryRequest.SerializeToString,
            proxy__index__pb2.VectorScanQueryResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VectorGetRegionMetrics(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/dingodb.pb.proxy.index.IndexService/VectorGetRegionMetrics',
            proxy__index__pb2.VectorGetRegionMetricsRequest.SerializeToString,
            proxy__index__pb2.VectorGetRegionMetricsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



from dingodb.protos.proxy_common_pb2 import *
from google.protobuf.json_format import MessageToDict

metric_type = {
    "euclidean": METRIC_TYPE_L2,
    "dotproduct": METRIC_TYPE_INNER_PRODUCT,
    "cosine": METRIC_TYPE_COSINE,
}

# proto
index_config = {
    "flat": MessageToDict(
        VectorIndexParameter(
            flat_parameter=CreateFlatParam(dimension=None, metric_type=None),
            vector_index_type=VECTOR_INDEX_TYPE_FLAT,
        ),
        including_default_value_fields=True,
    ),
    "ivf_flat": MessageToDict(
        VectorIndexParameter(
            ivf_flat_parameter=CreateIvfFlatParam(
                dimension=None, metric_type=None, ncentroids=256
            ),
            vector_index_type=VECTOR_INDEX_TYPE_IVF_FLAT,
        ),
        including_default_value_fields=True,
    ),
    "ivf_pq": MessageToDict(
        VectorIndexParameter(
            ivf_pq_parameter=CreateIvfPqParam(
                dimension=None,
                metric_type=None,
                ncentroids=256,
                nsubvector=8,
                bucket_init_size=0,
            ),
            vector_index_type=VECTOR_INDEX_TYPE_IVF_PQ,
        ),
        including_default_value_fields=True,
    ),
    "hnsw": MessageToDict(
        VectorIndexParameter(
            hnsw_parameter=CreateHnswParam(
                dimension=None,
                efConstruction=200,
                max_elements=50000,
                metric_type=None,
                nlinks=32,
            ),
            vector_index_type=VECTOR_INDEX_TYPE_HNSW,
        ),
        including_default_value_fields=True,
    ),
    "diskann": MessageToDict(
        VectorIndexParameter(
            diskann_parameter=CreateDiskAnnParam(dimension=None, metric_type=None),
            vector_index_type=VECTOR_INDEX_TYPE_DISKANN,
        ),
        including_default_value_fields=True,
    ),
}
